/*---------------------------------------------------------------------------*/
/* File: FrFileChk.c                                               B. Mours  */
/*                                            Last update:     July 6, 2001  */
/*                                                                           */
/*  This program reads and check a frame file. In case of success it returns */
/*  a positive value which is the numberof frame in the file or a negative   */
/*  value in case of read (-1) or checksum error (-2).                       */
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "FrameL.h"

void Help()
{
 printf(" \n" 
     " This program reads and check all the frames in the file             \n"
     "-------------------------------------------------------------------- \n"
     " Syntax is: FrFileCheck  -i <input file>                             \n"
     "                    -d <debug level>                                 \n"
     "                    -h (to get this help)                            \n"
     "-------------------------------------------------------------------- \n"
     " Remarks:                                                            \n"
     "  -i : Argument should be only one file names                        \n"
     "-------------------------------------------------------------------- \n"
     "\n");
}

struct Parameters{
  char *name;                   /* input file name                  */
  int debug;                    /* debug level                      */
};

void ReadParameters(int argc, char **argv);

struct Parameters Par;

/*--------------------------------------------------------- Main ------------*/
int main(int argc, char **argv)
/*---------------------------------------------------------------------------*/
{FrFile *file;         
 FrameH *frame = NULL;
 int nFrame = 0, nPrint = 1;
 
           /*----------------- Read input arguments -------------------------*/

 ReadParameters(argc, argv);

           /*---------------------------- Open the input file(s) ------------*/

 FrLibSetLvl(Par.debug-1);
 file = FrFileINew(Par.name);
 if(file == NULL)
    {printf("Input file (%s) open error:\n %s", Par.name, FrErrorGetHistory());
     return(-1);}
 file->chkSumFlag = FR_YES;

           /*----------------Start the Main loop ----------------------------*/

 while((frame = FrameReadRecycle(file, frame)) != NULL) 
   {nFrame++;
    if(nFrame != nPrint) continue;
    nPrint *= 2;
    if(Par.debug) printf("%9d frames read\n",nFrame);}
 
 if(Par.debug > 0) printf("   %d frame(s) in file\n",nFrame);
 if(file->error != FR_OK)
      {printf("Read Error. Last errors are:\n%s",FrErrorGetHistory());
       return(-1);}
 else {if(Par.debug > 0) printf(" No read error\n");}
  
 if(file->chkFlagRead == 0)
   {if(Par.debug > 0) printf(" This file does not have a checksum\n");}
 else
   {if(file->chkSumRead == file->chkSum)
      {if(Par.debug > 0) printf(" Checksum OK (%x)\n",file->chkSumRead);}
    else
      {if(Par.debug > 0) printf(" Cheksum error:     read = %x\n"
                                "                computed = %x\n",
                                file->chkSumRead, file->chkSum);}
    return(-2);}
 
 exit(nFrame);
}


/*------------------------------------------------------ SetParameters ------*/
void ReadParameters(int argc, char **argv)
/*---------------------------------------------------------------------------*/
{int i;

 if (argc == 1)
    {Help();
     exit(0);}

 Par.name   = NULL;
 Par.debug  = 1;

           /*------------------- Loop over the parameters -------------------*/

 for (i=1; i<argc; i++)
    {if     (strcmp(argv[i],"-h") == 0) 
       {Help(); 
        exit(0);}
    else if (strcmp(argv[i],"-i") == 0) {FrStrCpy(&(Par.name),argv[i+1]);}
    else if (strcmp(argv[i],"-d") == 0) {Par.debug = atoi(argv[i+1]);}}

           /*-------------------------- Check parameters---------------------*/

 if(Par.name == NULL) 
      {printf(" Please provide at least one input file\n");
       Help();
       exit(0);}

 if (Par.debug >0) printf(" Test file %s\n  "
                   "  Scanning the file Please wait....\n", Par.name);

 return;
}
