/*-----------------------------------------------------------*/
/* File: exampleFull.c         by B. Mours(LAPP) Jun 20, 2001*/
/*                                                           */
/*  This program creates a complex frame and write it on file*/
/*-----------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "FrameL.h"
  
int main()     /*-------------------- main-----------*/
{FrFile *oFile;
 FrameH *frame;
 FrAdcData *adc;
 FrSimData *sim;
 FrTable *table;
 FrTrigData *trig;
 FrSimEvent *evt;
 FrVect *col;
 double sampleRate, phase;
 long nData,i,j,nADC,sec;
 char name[100];
 
    /*------ initialisation -------------------------------*/
 
  if(FrLibIni("full.lis",NULL, 2) == NULL)
     {printf("Error during initialisation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

    /*----------------------- Create a new frame ---------*/
    
  frame = FrameNew("demo");
  if(frame == NULL)
     {printf("FrameNew failed (%s)",FrErrorGetHistory());
      return(0);}

   /*----- add fasts (2kHz) ADC with data provided 
                          by a random generator ------------*/

  sampleRate = 2000.;
  nData = 2000;
  frame->dt = nData/sampleRate;  
  nADC = 2;
 
  phase = 0;
  for(i=0; i<nADC; i++)
     {sprintf(name,"fastAdc%ld",i);
      adc = FrAdcDataNew(frame,name,sampleRate,nData,16);
      if(adc == NULL) return(0);
      for(j=0; j < nData; j++)
         {phase += .04;
          adc->data->dataS[j] = (short) (rand()/200 + 200.*cos(phase));}}
  
  /*---- reserve space for slower (200 Hz) ADC's ------*/

  nData = nData / 10;
  sampleRate = sampleRate/10.;
  nADC = 4;
  for(i=0; i<nADC; i++)
     {sprintf(name,"Adc%ld",i);
      adc = FrAdcDataNew(frame,name,sampleRate,nData,16);

      for(j=0; j < adc->data->nData; j++)
         {adc->data->dataS[j] = (short) (rand());}}

  /*----- add one table -----------------------*/
 
  table = FrTableNew("table_1","slow monitoring",5,2,
               FR_VECT_STRING, "channel", FR_VECT_4R,"value");
  frame->rawData->firstTable = table;
     
  col = FrTableGetCol(table,"value");
  for(j=0; j < col->nData; j++) {col->dataF[j] = .2*cos(.1*j);}

    /*---------------- reserve space for various type of data --*/
  
  adc = FrAdcDataNew(frame,"Bad data (data[0]=nan)",sampleRate,nData,-32);
  for(j=0; j < nData; j++) {adc->data->dataF[j] = .1*j;}
  adc->data->dataUI[0] = 0x7f800000;            /*-- this is INF*/
  adc->data->dataUI[1] = 0x7f801000;            /*-- this is NAN*/

  adc = FrAdcDataNew(frame,"D1",sampleRate,nData,-32);
  for(j=0; j < nData; j++) {adc->data->dataF[j] = sin(.01*j);}
  
  /*-- We build two sim1 channel to check the errors messages --*/

  sim = FrSimDataNew(frame,"sim1",sampleRate,nData,32);
  sim = FrSimDataNew(frame,"sim1",sampleRate,nData,32);
  for(j=0; j < nData; j++) {sim->data->dataI[j] = j;}
  
  FrSerDataNew(frame,"sms1",0,"sms data are here");
  FrMsgAdd(frame,"Test","Test message",0);
  FrSummaryNew(frame,"Quality_1","main quality"," ",NULL, NULL);
  FrSummaryNew(frame,"Quality_2","aux"," ",NULL, NULL);
  
  FrTrigDataNew(frame,"trigger_1","Binary search","Main strain",
           0,     0,1.,.2,1.e-20,.01,"M1 1.4 M2 1.4",NULL);
  FrTrigDataNew(frame,"trigger_1","Binary search","Main strain",
           0,9000000,1.,.2,2.e-20,.01,"M1 234. M2 0.123",NULL);
  FrTrigDataNew(frame,"Burst_1","Burst algorithm 1234","Main strain",
           0,0,5.,.3,2.e-20,.02,NULL,NULL);
  FrSimEventNew(frame,"test event",NULL,"1.4-1.4",0, .24,.2,.1,1.e-21);  

    /*---------------- open the output file with  compression --*/

  oFile = FrFileONewH("test.dat", 5, "FrFull"); 
  if(oFile == NULL)
     {printf("Error during output file creation\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return(0);}

    /*------ ---------------------------------output 10 frames--*/
    
  for(i=0; i<10; i++)
     {printf(" output frame %d\n",frame->frame);
 
      trig = NULL;
      while((trig = FrTrigDataFind(frame,"trigger_1", trig)) != NULL)
       {trig->GTimeS = frame->GTimeS;
        trig->GTimeN = .02*i;
        trig->amplitude = 1.1*trig->amplitude;}

      evt = FrSimEventFind(frame,"test event", NULL);
      evt->GTimeS = frame->GTimeS;  

      /*---------------------- dump on stdout the current frame--*/

      if(i < 2) FrameDump(frame, stdout,2);

      if(FrameWrite(frame, oFile) != FR_OK)
          {printf("Error during frame write\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
           return(0);}

      frame->frame++;
      frame->GTimeS += (unsigned int) (frame->dt);
      frame->GTimeN += ((unsigned int) (1.e+9*frame->dt)) % 1000000000;
      sec = frame->GTimeN/1000000000;
      frame->GTimeS += sec;
      frame->GTimeN -= sec*1000000000;

      adc = FrAdcDataFind(frame,"fastAdc1");
      if(adc != NULL)
        {for(j=0; j < adc->data->nData; j++)
          {phase += .04;
           adc->data->dataS[j] = (short) (rand()/200 + 200.*cos(phase));}}
}
  
   /*------ try the tag function ---------------------*/
  
  printf("\n>>>>>>>>> Now dump frame with tag=sim* *1\n");
  FrameTag(frame, "sim* *1");
  FrameDump(frame, stdout, 2);

  printf("\n>>>>>>>>> Now all channels except trig channels \n");
  FrameTag(frame, "-trig*");
  FrameDump(frame, stdout, 2);

    /*----- close everything -------------------------*/

  FrFileOEnd(oFile);
 
  FrameFree(frame);

  return(0);
}
